/**
 * C++ Class    TCPFlow
 *
 * Revision     1.0a (9th August 2010)
 * Programmer   Michael Allwright
 *
 * Purpose      Extention to the STL Vector container class, which has been designed to hold parameters belonging to a
 *              given flow. Class also contains some metadata about the flow, as well as methods to 'grab' a specific
 *              parameter over a given time period.
 *              
 **/

#include <vector>
#include <sstream>

#include "tcpflow.h"
#include "tcpparams.h"

/**
 * Method Name  std::ostream & operator<<(std::ostream & theStream, const TcpFlow & theFlow)
 *
 * Parameters   std::ostream & theStream - An ostream object like cout, cerr, or a file/string stream
 *              const TcpFlow & theFlow  - The flow to be outputted
 *
 * Returns      Reference to the given stream from parameter 1.
 *
 * Description  Stream extraction method - provides '<<' access to the flow ID
**/
std::ostream & operator<<(std::ostream & theStream, const TcpFlow & theFlow)
{
    return (theStream << theFlow.tcpFlowID);
}

/**
 * Method Name  TcpFlow::TcpFlow(const TcpParams & parameters)
 *
 * Parameters   const TcpParams & parameters - A TcpParams structure for initialisation
 *
 * Description  Constructor
**/
TcpFlow::TcpFlow(const TcpParams & parameters)
{
    tcpFlowID = parameters.tcpFlowID;
    firstPacketTimestamp = parameters.packetTimeStamp;
    lastPacketTimestamp = parameters.packetTimeStamp;
    events.push_back(parameters);
}

/**
 * Method Name  void TcpFlow::appendStatsToFlow(const TcpParams & toAppend)
 *
 * Parameters   const TcpParams & toAppend - The TcpParams structure to be appended to the flow
 *
 * Description  Method for adding a set of parameters to this flow.
**/
void TcpFlow::appendStatsToFlow(const TcpParams & toAppend)
{
    lastPacketTimestamp = toAppend.packetTimeStamp;
    events.push_back(toAppend);
}


/**
 * Method Name  std::vector<unsigned int> TcpFlow::getParameterList(int flowChar, double from, double to) const
 *
 * Parameters   int flowChar - The flow characteristic to be returned
 *              double from  - timestamp to start gathering data from.
 *              double to    - timestamp to conclude gathering data at.
 *
 * Description  Returns a list of a given parameter over the interval (from, to)
**/
std::vector<unsigned int> TcpFlow::getParameterList(int flowChar, double from, double to) const
{
    std::vector<unsigned int> theList;

    for(std::vector<TcpParams>::const_iterator it = events.begin(); it < events.end(); it++)
    {
    if((it->packetTimeStamp > from) and (it->packetTimeStamp < to))
        theList.push_back(it->parameter[flowChar]);
    }
    return theList;
}

/**
 * Method Name  std::vector<unsigned int> TcpFlow::getParameterList(int flowChar, double from) const
 *
 * Parameters   int flowChar - The flow characteristic to be returned
 *              double from  - timestamp to start gathering data from.
 *
 * Description  Returns a list of a given parameter over the interval (from, END_OF_DATA)
**/
std::vector<unsigned int> TcpFlow::getParameterList(int flowChar, double from) const
{
    std::vector<unsigned int> theList;

    for(std::vector<TcpParams>::const_iterator it = events.begin(); it < events.end(); it++)
    {
    if(it->packetTimeStamp > from)
        theList.push_back(it->parameter[flowChar]);
    }
    return theList;
}

/**
 * Method Name  std::vector<unsigned int> TcpFlow::getParameterList(int flowChar) const
 *
 * Parameters   int flowChar - The flow characteristic to be returned
 *              
 * Description  Returns a list of a given parameter (from all records)
**/
std::vector<unsigned int> TcpFlow::getParameterList(int flowChar) const
{
    std::vector<unsigned int> theList;

    for(std::vector<TcpParams>::const_iterator it = events.begin(); it < events.end(); it++)
    theList.push_back(it->parameter[flowChar]);

    return theList;
}
